﻿//********************************************************************************
// Name:Ai_continuous.cs
// Author: Smacq
// Copyright: Smacq
// Date: 22-11-2024
// Brief:
// The Ai functions of 1252A_DAQ, continuous acquisition.
// Acquisition Ai0 channel 100 data point of 10 times.
// You can loot out user manual know more about functions or Device info.

// 1252A_DAQ only connect USB execution result:
// 0
// 0
// 0
// 0
// 0
// 0
// 0
// 0
// Start acquisition times:0 Ai_data_array[0-99]:
// FIFO buffer value:1999995
// Ai_data_array[0]:-3.481503
// Ai_data_array[1]:-3.481503
// ...
// Ai_data_array[99]:-3.476113
// Start acquisition times:1 Ai_data_array[0-99]:
// FIFO buffer value:1999989
// Ai_data_array[0]:-3.478808
// ...
// Start acquisition times:9 Ai_data_array[0-99]:
// ...
// Ai_data_array[99]:-3.476113
// 0
// 0

// Error code:
// Execution successful  =  0
// Device not found      = -1
// DevIndex overflow     = -2
// Device firmware error = -3
// Device closed         = -4
// Transfer data error   = -5
// PC Out of memory      = -6
// Execution timeout     = -7
// Reading thread not on = -8
//********************************************************************************

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace USB_1000_AI_Continuous
{
    class Program
    {
        static void Main(string[] args)
        {

            int Dev_Index = 0;
            int error_code, buffer, temp;
            byte[] SN = new byte[8];
            float[] Ai_data_array = new float[1600];

            error_code = usb1000.OpenDevice(Dev_Index);           //Turn on the specified device.
            Console.WriteLine(error_code);
            error_code = usb1000.ResetDevice(Dev_Index);          //Reset the specified device.
            Console.WriteLine(error_code);
            error_code = usb1000.SetUSB1AiRange(Dev_Index, 5);    //Set Device Ai input voltage range 5 = -5~5V or 10 = 0~10V
            Console.WriteLine(error_code); 
            error_code = usb1000.SetSampleRate(Dev_Index, 1000);  //Set Ai Sampling rate 1kS/s
            Console.WriteLine(error_code); 
            error_code = usb1000.SetChanMode(Dev_Index, 1);       //Set Device Ai input channel mode 0=diff or 1=NRSE or 3=RSE
            Console.WriteLine(error_code);
            /*
             * Set Device Ai0 channel open
             * 1=0x0001=16-bit(0000 0000 0000 0001) Ai15~1 is CLOSE and Ai0 is OPEN.
             * 3=0x0003=16-bit(0000 0000 0000 0011) Ai15~2 is CLOSE and Ai1~0 is OPEN.
             * 65535=0xFFFF=16-bit(1111 1111 1111 1111) Ai15~0 is OPEN.
             */
            error_code = usb1000.SetChanSel(Dev_Index, 1);
            Console.WriteLine(error_code); 
            error_code = usb1000.StartRead(Dev_Index);           //Turn on Device reading thread      
            Console.WriteLine(error_code); 
            error_code = usb1000.SetSoftTrig(Dev_Index, 1);      //Turn on Device software trigger
            Console.WriteLine(error_code);

            //Continuous acquisition execution 10 times
            for (int i = 0; i < 10; i++)
            {
                /* 
                 * Ai_FIFO_buffer range 0~200000
                 * GetAiChans() return the remaining Device FIFO buffer value, 
                 * FIFO buffer 100 data points start save to Ai_data_array_ptr is Ai_data_array[0:100]
                 * Num: Each opened Ai channel will get this amount of data.
                 * Suggest setting this value within sampling rate 10-20 %.
                 * Timeout time unit ms, Suggest timeout>(Num*Channel_count/samplerate)*1000.
                 * Avoid GetAiChans() func timeout return -7.
                 */
                buffer = usb1000.GetAiChans(Dev_Index, 100, 1, Ai_data_array, 200);
                Console.WriteLine("FIFO buffer value:{0}", buffer);

                //Print Ai_data_array to Console 
                Console.WriteLine("Start acquisition times:{0} Ai_data_array[0-99]:", i);
                for (int j = 0; j < 100; j++)
                   Console.WriteLine("Ai_data_array[{0}]:{1}", j, Ai_data_array[j]);
                //Add your code here handle data
            }

            error_code = usb1000.SetSoftTrig(Dev_Index, 0); //Turn off Device software trigger
            Console.WriteLine(error_code);
            error_code = usb1000.StopRead(Dev_Index);	    //Turn off Device reading thread
            Console.WriteLine(error_code);
            usb1000.CloseDevice(Dev_Index);                 //Close Device
            Console.ReadLine();                             //Delay for show result
        }
        class usb1000
        {
            [DllImport("usb1000.dll")]
            public static extern int FindUSBDAQ();
            [DllImport("usb1000.dll")]
            public static extern int OpenDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int USB1GetDeviceSN(int DevIndex, byte[] SN);
            [DllImport("usb1000.dll")]
            public static extern int ResetDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern void CloseDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int SetUSB1AiRange(int DevIndex, float Range);
            [DllImport("usb1000.dll")]
            public static extern int SetSampleRate(int DevIndex, uint SampleRate);
            [DllImport("usb1000.dll")]
            public static extern int SetChanSel(int DevIndex, UInt16 ChSel);
            [DllImport("usb1000.dll")]
            public static extern int SetChanMode(int DevIndex, byte ChanMode);
            [DllImport("usb1000.dll")]
            public static extern int SetTrigSource(int DevIndex, byte TrigSource);
            [DllImport("usb1000.dll")]
            public static extern int SetSoftTrig(int DevIndex, byte Trig);
            //public static extern int SetTrigEdge(int DevIndex, byte TrigEdge);
            [DllImport("usb1000.dll")]
            public static extern int ClearTrigger(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int SetDioOut(int DevIndex, int DioOut);
            [DllImport("usb1000.dll")]
            public static extern int GetDioIn(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int TransDioIn(int DevIndex, byte TransDioSwitch);

            //public static extern int SetCounter(int DevIndex, byte CtrNum, byte CtrMode, byte CtrEdge);

            // public static extern int StartCounter(int DevIndex, byte CtrNum, byte OnOff);

            //public static extern int InitDA(int DevIndex);

            //public static extern int SetDA(int DevIndex, byte DANum, float DAVolt);

            //public static extern int SetWavePt(int DevIndex, byte DANum, float DAVolt);

            //public static extern int ClrWavePt(int DevIndex, byte DANum);

            //public static extern int SetWaveSampleRate(int DevIndex, uint WaveSampleRate);

            // public static extern int WaveOutput(int DevIndex, byte DANum);

            [DllImport("usb1000.dll")]
            public static extern int ClearBufs(int DevIndex);
            //public static extern int ClearCounter(int DevIndex, byte CtrNum);
            [DllImport("usb1000.dll")]
            public static extern int StartRead(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int StopRead(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int GetAiChans(int DevIndex, int Num, UInt16 ChSel, float[] Ai, int TimeOut);
        }
    }
}