﻿//********************************************************************************
// Name:Counter.cs
// Author: Smacq
// Copyright: Smacq
// Date: 25-11-2024
// Brief:
// The Counter functions of 1252A_DAQ, get 10s Ct0-3 signal cycle value and print it.
// You can loot out user manual know more about functions or Device info.
// 1252A_DAQ only connect USB execution result:
// 0
// 0
// 0
// 0
// 0
// 0
// 0 second:
// Ct_channel: 0，Ct_counter: 1
// Ct_channel: 1，Ct_counter: 1
// Ct_channel: 2，Ct_counter: 1
// Ct_channel: 3，Ct_counter: 1
// ...
// 9 second:
// Ct_channel: 0，Ct_counter: 1
// Ct_channel: 1，Ct_counter: 1
// Ct_channel: 2，Ct_counter: 1
// Ct_channel: 3，Ct_counter: 1
// 0
// 0
// 0

// Error code:
// Execution successful  =  0
// Device not found      = -1
// DevIndex overflow     = -2
// Device firmware error = -3
// Device closed         = -4
// Transfer data error   = -5
// PC Out of memory      = -6
// Execution timeout     = -7
// Reading thread not on = -8
//********************************************************************************

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

namespace Counter
{
    class Program
    {
        static void Main(string[] args)
        {
            int Dev_Index = 0;
            int ct_count, error_code;
            byte CtrNum = 15;
            byte CtrMode = 0;                                 
            byte CtrEdge = 1;
            double ct_time;

            error_code = usb1000.OpenDevice(Dev_Index);          //Turn on the specified device.
            Console.WriteLine(error_code);
            error_code = usb1000.ResetDevice(Dev_Index);         //Reset the specified device.
            Console.WriteLine(error_code);
            error_code = usb1000.StartRead(Dev_Index);           //Turn on Device reading thread      
            Console.WriteLine(error_code);
            error_code = usb1000.TransDioIn(Dev_Index, 1);       //Turn on Device Digital IO Transfer thread
            Console.WriteLine(error_code);

            /*
             * Set Device Counter: Device_0, CrtNum_Ct0-3, CrtMode_1(cycle)
             * CtrNum:1=0x1=4-bit(0001) Ct3~1 is CLOSE and Ct0 is OPEN.15=0xF=4-bit(1111) Ct3~0 is OPEN.
             * CtrMode:0=edge or 1=cycle or 2=positive pulse width or 3=negative pulse width 1252A only support param 0~3
             * CtrEdge:1=rising or 2=falling 1252A only support param 1/2 
             */
            error_code = usb1000.SetCounter(Dev_Index, CtrNum, CtrMode, CtrEdge);
            Console.WriteLine(error_code);
            error_code = usb1000.StartCounter(Dev_Index, CtrNum, 1);  //Start Device Counter record
            Console.WriteLine(error_code);

            //Run 10s for catch Ct0~3 signal, each second update input signal value and print
            for (int i = 0; i < 10; i++)
            {
                Thread.Sleep(1000);                                    //delay 1s
                Console.WriteLine("{0} second:", i);
                //Get Ct0~3 signal cycle value and print
                for (byte ct_channel = 0; ct_channel < 4; ct_channel++)                           
                {
                    //GetCounter() only support CrtMode=0(edge), return edge count
                    if (CtrMode == 0)
                    {
                        ct_count = usb1000.GetCounter(Dev_Index, ct_channel);           
                        Console.WriteLine("Ct_channel:{0}，Ct_count:{1}", ct_channel, ct_count);
                    }
                    //GetCtrTime() not support CrtMode=0(edge), return value is time unit us
                    else
                    {
                        ct_time = usb1000.GetCtrTime(Dev_Index, ct_channel);
                        Console.WriteLine("Ct_channel:{0}，Ct_time:{1}", ct_channel, ct_time);
                    }
                }
                
            }

            error_code = usb1000.StartCounter(Dev_Index, CtrNum, 0);  //Stop Device Counter record
            Console.WriteLine(error_code);
            error_code = usb1000.ClearCounter(Dev_Index, CtrNum);     //Clear Ct0-3 count value
            Console.WriteLine(error_code);
            error_code = usb1000.TransDioIn(Dev_Index, 0);            //Turn off Device Digital IO transfer thread
            Console.WriteLine(error_code);
            error_code = usb1000.StopRead(Dev_Index);	              //Turn off Device reading thread
            Console.WriteLine(error_code);
            usb1000.CloseDevice(Dev_Index);                           //Close Device

            Console.ReadLine();                                       //Delay for show result
        }

        class usb1000
        {
            [DllImport("usb1000.dll")]
            public static extern int OpenDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int ResetDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern void CloseDevice(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int GetCounter(int DevIndex, byte CtrNum);
            [DllImport("usb1000.dll")]
            public static extern double GetCtrTime(int DevIndex, byte CtrNum);
            [DllImport("usb1000.dll")]
            public static extern int TransDioIn(int DevIndex, byte TransDioSwitch);
            [DllImport("usb1000.dll")]
            public static extern int SetCounter(int DevIndex, byte CtrNum, byte CtrMode, byte CtrEdge);
            [DllImport("usb1000.dll")]
            public static extern int StartCounter(int DevIndex, byte CtrNum, byte OnOff);
            [DllImport("usb1000.dll")]
            public static extern int ClearCounter(int DevIndex, byte CtrNum);
            [DllImport("usb1000.dll")] 
            public static extern int StartRead(int DevIndex);
            [DllImport("usb1000.dll")]
            public static extern int StopRead(int DevIndex);
        }
    }
}
